
//-----------------------------------------------------------------------------
// Purpose: clears m_hMyWeapons from any rogue parachutes being detected by other functions
//-----------------------------------------------------------------------------

::GTFW.ClearInvalidItemsInLoadout <- function(player)
{
	for (local i = 0; i < 47; i++)
	{
		local wep = GetPropEntityArray(player, "m_hMyWeapons", i);
		
		if ( wep == null ) continue
		if ( wep.GetClassname() == "null" ) SetPropEntityArray(player, "m_hMyWeapons", null, i);
	}
}


//-----------------------------------------------------------------------------
// Purpose: Creates a tf_wearable
//-----------------------------------------------------------------------------

::CTFPlayer.CreateCustomWearable <- function(item_modelstring="models/empty.mdl",itemID=65535, weapon=null)
{
	GTFW.DevPrint(0,"CreateCustomWearable", "Making \x22"+item_modelstring+"\x22 with ID "+ itemID);

	local ply = this;

//spawns a legitimate, verified tf_wearable (the parachute), then changes the model.
//this allows it to stay on the ragdoll on death (among some other perks I'm sure)
//credits ficool2
    local dummy_Weapon = Entities.CreateByClassname("tf_weapon_parachute");
    SetPropInt(dummy_Weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 1101);
    SetPropBool(dummy_Weapon, "m_AttributeManager.m_Item.m_bInitialized", true);
	dummy_Weapon.SetTeam(ply.GetTeam());
    dummy_Weapon.DispatchSpawn(); 
    ply.Weapon_Equip(dummy_Weapon);
	dummy_Weapon.KeyValueFromString("classname","null");
    
    local hWearable = GetPropEntity(dummy_Weapon, "m_hExtraWearable");
	
    dummy_Weapon.Kill();
	GTFW.ClearInvalidItemsInLoadout(ply);
	
	if ( type( item_modelstring ) == "string" ) {
		PrecacheModel(item_modelstring);
		hWearable.SetModelSimple(item_modelstring);
	}
	else if ( type( item_modelstring ) == "integer" )
		SetPropInt(hWearable, "m_nModelIndex", item_modelstring);
	else if ( type(itemID) == "integer" ) {
		item_modelstring = GetItemModelName(itemID);
		PrecacheModel(item_modelstring);
		hWearable.SetModelSimple(item_modelstring);
	}
	
	if ( type(itemID) == "integer" )
		SetPropInt(hWearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemID); 
	else		
		SetPropInt(hWearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 65535);
		
    SetPropBool(hWearable, "m_AttributeManager.m_Item.m_bInitialized", true);
	
    // (optional) recalculates bodygroups on the player
	//GTFW: We add "early_out" to fix ent overload crash while keeping this feature
    SendGlobalGameEvent("post_inventory_application", { userid = GTFW_GetPlayerUserID(ply), early_out = true });


// GiveWeapon: If creating a wearable for a weapon, always delete the extra wearable so it doesn't stick to the player forever
	if ( weapon && weapon.IsValid() 
	  && HasProp(weapon, "m_hExtraWearable")
	) {
		local extra_wearable = GetPropEntity(weapon, "m_hExtraWearable");
		if ( extra_wearable && extra_wearable.IsValid() ) extra_wearable.Kill();
		
		weapon.ValidateScriptScope();
		local weapon_scope = weapon.GetScriptScope();
		if ( !( "GTFW_wearables_on_weapon" in weapon_scope ) ) weapon_scope.GTFW_wearables_on_weapon <- [];
		weapon_scope.GTFW_wearables_on_weapon.append(hWearable);
	}
	
	
	
// For cleanup, we shove everything into the player's scope for easy collection
	ply.ValidateScriptScope();
	local ply_scope = ply.GetScriptScope();
	if ( !( "GTFW_wearables" in ply_scope ) ) ply_scope.GTFW_wearables <- [];
	ply_scope.GTFW_wearables.append(hWearable);
	
	GTFW.DevPrint(0,"CreateCustomWearable", "Created "+"\x22"+item_modelstring+"\x22 with ID "+itemID);

	return hWearable;
}



//-----------------------------------------------------------------------------
// Purpose: Creates a tf_wearable for weapon's world model only
//-----------------------------------------------------------------------------

::CTFPlayer.CreateCustomWearableWeaponWorldModel <- function(wearable_classname="tf_wearable",item_modelstring="models/empty.mdl",itemID=65535,weapon=null)
{
	GTFW.DevPrint(0,"CreateCustomWearableWeaponWorldModel", "Making \x22"+wearable_classname+"\x22 for "+weapon);

	local ply = this;
	local item_modelindex = GetModelIndex("models/empty.mdl");
	local main_viewmodel = GetPropEntity(ply, "m_hViewModel");
	
// Defining which entity should be created
	//Only these classnames are valid:
	switch ( wearable_classname ) {
		case "tf_wearable" : break;
	//	case "tf_wearable_vm" : break;
		case "tf_wearable_demoshield" : break;
		case "tf_wearable_razorback" : break;
		case "tf_weapon_parachute" : break;
		case "tf_weapon_parachute_primary" : break;
		case "tf_weapon_parachute_secondary" : break;
		default : wearable_classname = "tf_wearable"; break;
	}


// Precaches weapon if string
	if ( type( item_modelstring ) == "string" )
	{
		PrecacheModel(item_modelstring);
		item_modelindex = GetModelIndex(item_modelstring);
	}
	else {
		if ( weapon && weapon.IsValid() ) itemID = GetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex"); 
		item_modelstring = GetItemModelName(itemID);
		PrecacheModel(item_modelstring);
		item_modelindex = GetModelIndex(item_modelstring);
	}
	
// Creates the wearable that does not parent itself to ragdoll
    local hWearable = Entities.CreateByClassname(wearable_classname);
	
	hWearable.SetAbsAngles(QAngle(0,0,0))
	SetPropInt(hWearable, "m_fEffects", 161)
	hWearable.SetSolidFlags(4);
	hWearable.SetCollisionGroup(11);

	if ( type(itemID) == "integer" )
		SetPropInt(hWearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemID); 
	else
		SetPropInt(hWearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 65535);
    SetPropBool(hWearable, "m_AttributeManager.m_Item.m_bInitialized", true);
	SetPropBool(hWearable, "m_bValidatedAttachedEntity", true);
	SetPropBool(hWearable, "m_AttributeManager.m_iReapplyProvisionParity", true);
	SetPropBool(hWearable, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", true)	//Wipe attributes
	
	hWearable.SetTeam(ply.GetTeam());
    hWearable.DispatchSpawn(); 
	
//	Makes sure our weapons reappear appropriately while equipped, and disappear when not
	SetPropEntity(hWearable, "m_hOwnerEntity", ply);
	hWearable.SetOwner(ply);
	hWearable.ReapplyProvision();		//reapplies any attributes from weapon onto the player

	if ( type( item_modelstring ) == "string" )
		hWearable.SetModelSimple(item_modelstring);
	else {
		SetPropInt(hWearable, "m_nModelIndex", item_modelindex);
		hWearable.KeyValueFromInt("modelindex", item_modelstring);
	}

    // (optional) recalculates bodygroups on the player
	//GTFW: We add "early_out" to fix ent overload crash while keeping this feature
    SendGlobalGameEvent("post_inventory_application", { userid = GTFW_GetPlayerUserID(ply), early_out = true });


	if ( weapon && weapon.IsValid() && weapon != main_viewmodel ) {
// Used by AddThinkToView()'s think script to find the prop
		weapon.ValidateScriptScope();
		local weapon_scope = weapon.GetScriptScope();
		local hasWorldModelProp = "worldModelProp" in weapon_scope ? weapon_scope.worldModelProp : null;
		if ( hasWorldModelProp && hasWorldModelProp.IsValid() ) hasWorldModelProp.Kill();
		weapon_scope.worldModelProp <- hWearable;
		
		if ( !( "GTFW_wearables_on_weapon" in weapon_scope ) ) weapon_scope.GTFW_wearables_on_weapon <- [];
		weapon_scope.GTFW_wearables_on_weapon.append(hWearable);
		
//	Makes sure our weapons reappear appropriately while equipped, and disappear when not
		DoEntFire("!self", "SetParent", "!activator", 0, ply, hWearable);
		SetPropEntity(weapon, "m_hExtraWearable", hWearable);
	}
	
	if ( wearable_classname != "tf_wearable" ) {
		ply.Weapon_Equip(hWearable);
	}

// For cleanup, we shove everything into the player's scope for easy disposing
	ply.ValidateScriptScope();
	local ply_scope = ply.GetScriptScope();
	if ( !( "GTFW_wearables" in ply_scope ) ) ply_scope.GTFW_wearables <- [];
	ply_scope.GTFW_wearables.append(hWearable);
	
	GTFW.DevPrint(0,"CreateCustomWearableWeaponWorldModel", "Created "+"\x22"+wearable_classname+"\x22 ("+item_modelstring+")");

	return hWearable;
}



//-----------------------------------------------------------------------------
// Purpose: Creates a tf_wearable_vm for weapon's view models only
//-----------------------------------------------------------------------------

::CTFPlayer.CreateCustomWearableViewModel <- function(item_modelstring="models/empty.mdl",weapon=null)
{
	GTFW.DevPrint(0,"CreateCustomWearableViewModel", "Making \x22"+item_modelstring+"\x22 for "+weapon);
	if ( type( item_modelstring ) != "string" && type( item_modelstring ) != "integer" )  {
		GTFW.DevPrint(1,"CreateCustomWearableViewModel", "Param 1 not a string or integer! Returning null");
		return null
	}

	local ply = this;
	local main_viewmodel = GetPropEntity(ply, "m_hViewModel");
	local itemID = GetItemID(weapon);
	
	if ( type( itemID ) == "integer" ) {
		local dummyID = itemID;
		for ( local bit = 16; bit <= 21; bit++ ) {
			dummyID = dummyID & ~( 1 << bit );
		}
		switch ( dummyID ) {
			case 131 : item_modelstring = "models/weapons/c_models/c_targe/c_targe.mdl"; break;	//Chargin' Targe
			case 406 : item_modelstring = "models/workshop/weapons/c_models/c_persian_shield/c_persian_shield.mdl"; break;	//Splendid Screen
			case 1099 : item_modelstring = "models/workshop/weapons/c_models/c_wheel_shield/c_wheel_shield.mdl"; break;	//Tide Turner
			case 1144 : item_modelstring = "models/weapons/c_models/c_targe/c_targe_xmas.mdl"; break;	//Festive Chargin' Targe
		}
	}


	local hWearable = Entities.CreateByClassname("tf_wearable_vm");

	if ( type( item_modelstring ) == "string" ) {
		PrecacheModel(item_modelstring);
		hWearable.SetModelSimple(item_modelstring);
	}
	else {
		SetPropInt(hWearable, "m_nModelIndex", item_modelstring);
		hWearable.KeyValueFromInt("modelindex", item_modelstring);
	}

	hWearable.SetTeam(ply.GetTeam());
	hWearable.DispatchSpawn();	//Spawns ent into world
	ply.EquipWearableViewModel(hWearable);
	
	if ( weapon && weapon != main_viewmodel ) {
		SetPropEntity(hWearable, "m_hWeaponAssociatedWith", weapon);
		SetPropEntity(weapon, "m_hExtraWearableViewModel", hWearable);
		hWearable.ReapplyProvision();		//reapplies any attributes from weapon onto the player

// Purpose: AddThinkToView()
		weapon.ValidateScriptScope();
		local weapon_scope = weapon.GetScriptScope();
		local hasViewModelProp = "viewModelProp" in weapon_scope ? weapon_scope.viewModelProp : null;
		if ( hasViewModelProp && hasViewModelProp.IsValid() ) hasViewModelProp.Kill();
		weapon_scope.viewModelProp <- hWearable;
		
		if ( !( "GTFW_wearables_on_weapon" in weapon_scope ) ) weapon_scope.GTFW_wearables_on_weapon <- [];
		weapon_scope.GTFW_wearables_on_weapon.append(hWearable);
	}
	else if ( weapon == main_viewmodel ) {
		hWearable.ValidateScriptScope();
		local wearable_scope = hWearable.GetScriptScope();
		wearable_scope.model_path <- item_modelstring;
	}

// For cleanup, we shove everything into the player's scope for easy disposing
	ply.ValidateScriptScope();
	local ply_scope = ply.GetScriptScope();
	if ( !( "GTFW_wearables" in ply_scope ) ) ply_scope.GTFW_wearables <- [];
	ply_scope.GTFW_wearables.append(hWearable);
	
	GTFW.DevPrint(0,"CreateCustomWearableViewModel", "Created "+"\x22"+hWearable+"\x22 ("+item_modelstring+")");

	return hWearable;
}

//-----------------------------------------------------------------------------
// Purpose: Creates an econ item
//-----------------------------------------------------------------------------

::CTFPlayer.GiveEconItem <- function(itemID=65535, model=null)
{
	GTFW.DevPrint(0,"CreateCustomWearable", "Making ID "+ itemID);

	local ply = this;

//spawns a legitimate, verified tf_wearable (the parachute), then changes the model.
//this allows it to stay on the ragdoll on death (among some other perks I'm sure)
//credits ficool2
    local dummy_Weapon = Entities.CreateByClassname("tf_weapon_parachute");
    SetPropInt(dummy_Weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 1101);
    SetPropBool(dummy_Weapon, "m_AttributeManager.m_Item.m_bInitialized", true);
	dummy_Weapon.SetTeam(ply.GetTeam());
    dummy_Weapon.DispatchSpawn(); 
    ply.Weapon_Equip(dummy_Weapon);
	dummy_Weapon.KeyValueFromString("classname","null");
    
    local hWearable = GetPropEntity(dummy_Weapon, "m_hExtraWearable");
	
    dummy_Weapon.Kill();
	GTFW.ClearInvalidItemsInLoadout(ply);
	
	if ( type(itemID) == "integer" )
		SetPropInt(hWearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemID); 
	else		
		SetPropInt(hWearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 65535);
	
    SetPropBool(hWearable, "m_AttributeManager.m_Item.m_bInitialized", true); //required; otherwise it doesn't show up
    hWearable.DispatchSpawn();	//required; refreshes item definition's model 
	
	if ( type(model) == "string" )
		hWearable.SetModelSimple(model);
    // (optional) recalculates bodygroups on the player
	//GTFW: We add "early_out" to fix ent overload crash while keeping this feature
    SendGlobalGameEvent("post_inventory_application", { userid = GTFW_GetPlayerUserID(ply), early_out = true });

	
// For cleanup, we shove everything into the player's scope for easy collection
	ply.ValidateScriptScope();
	local ply_scope = ply.GetScriptScope();
	if ( !( "GTFW_wearables" in ply_scope ) ) ply_scope.GTFW_wearables <- [];
	ply_scope.GTFW_wearables.append(hWearable);
	
	GTFW.DevPrint(0,"CreateCustomWearable", "Created econ item ID "+itemID);

	return hWearable;
}