//-----------------------------------------------------------------------------
// Purpose: Finds a weapon based on handle, slot, ID, classnames, or string.
// 			Requires integer (slot, item definition index), or string (weapon name or classname).
//-----------------------------------------------------------------------------

::CTFPlayer.GetWeaponTableBySlot <- function(slot=TF_WEAPONSLOTS.SLOT0)
{
	if ( slot == null || type ( slot ) != "integer" ) {
		GTFW.DevPrint(1,"GetWeaponTableBySlot", "Function failed. Parameter is Null. Returning null.");
		return null;
	}
	else if ( slot > -0.0 ) {
		GTFW.DevPrint(1,"GetWeaponTableBySlot", "Function failed. Parameter is above 0. Returning null.");
		return null;
	}
	
	local ply = this;
	local baseitem;
	
// Purpose: Begins to search by slot
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = GetPropEntityArray(ply, "m_hMyWeapons", i);
		if ( wep == null ) continue
			if ( GetWeaponBySlotBool(wep, slot) )
			{
				local ItemID = GetItemID(wep);
				baseitem = GetWeaponTableNoPlayer(ItemID);
				baseitem = GTFW.PostWepFix(baseitem,ply);
				break;
			}
	}

	if ( !baseitem ) {
		GTFW.DevPrint(1,"GetWeaponTableBySlot", "Function failed. Could not find weapon by slot. Returning null.");
		return null;
	}
	
	return baseitem
}

::CTFPlayer.GetWeaponTableByClassname <- function(WepC=null)
{
	if ( WepC == null || type ( WepC ) != "string" ) {
		GTFW.DevPrint(1,"GetWeaponTableByClassname", "Function failed. Parameter is Null. Returning null.");
		return null;
	}
	
	local ply = this;
	local baseitem;
	
// Purpose: Searches by classname.
// 			Runs a loop to check all entities under array netprop "m_hMyWeapons"
	 if ( type( WepC ) == "string"
	 && SearchAllWeapons.find( WepC.find("weapon") ? WepC.slice(10) : WepC ) ) {	// makes sure to remove "tf_weapon_" if it's there
		WepC = WepC.find("weapon") ? WepC : "tf_weapon_" + WepC;				// makes sure "tf_weapon_" is added if it wasn't there before

		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = GetPropEntityArray(ply, "m_hMyWeapons", i);

			if ( wep )
			{
				if ( wep.GetClassname() == WepC )
				{
					local ItemID = GetItemID(wep);
					baseitem = GetWeaponTableNoPlayer(ItemID);
					baseitem = GTFW.PostWepFix(baseitem, ply);
					break;
				}
			}
		}
	}

	if ( !baseitem ) {
		GTFW.DevPrint(1,"GetWeaponTableByClassname", "Function failed. Could not find weapon by classname. Returning null.");
		return null;
	}
	
	return baseitem
}

::CTFPlayer.GetWeaponTableByString <- function(string=null)
{
	if ( string == null || type ( string ) != "string" ) {
		GTFW.DevPrint(1,"GetWeaponTableByString", "Function failed. Parameter is Null. Returning null.");
		return null;
	}
	
// Purpose: Catches item names (i.e. "Brass Beast" or "Wrench", etc)
	local baseitem = GetWeaponTableNoPlayer(string);
	
	if ( !baseitem ) {
		GTFW.DevPrint(1,"GetWeaponTableByString", "Function failed. Could not find weapon by name. Returning null.");
		return null;
	}
	
	baseitem = GTFW.PostWepFix(baseitem,this)
	return baseitem
}

::CTFPlayer.GetWeaponTableByHandle <- function(hWeapon=null)
{
	if ( type ( hWeapon ) != "instance" ) {
		GTFW.DevPrint(1,"GetWeaponTableByHandle", "Function failed. Parameter is not an instance. Returning null.")
		return null
	}
	
	local ItemID;
	if ( HasProp(hWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex") ) {
		ItemID = GetItemID(hWeapon)
	}
	else  {
		GTFW.DevPrint(1,"GetWeaponTableByHandle", "Function failed. Item doesn't have Item Definition Index. Returning null.")
		return null
	}
	
	// this one catches item IDs if we are using a handle
	local baseitem = GetWeaponTableNoPlayer(ItemID)
	
	if ( !baseitem ) {
		GTFW.DevPrint(1,"GetWeaponTableByHandle", "Function failed. Could not find weapon by Item Definition Index. Returning null.")
		return null
	}
	
	baseitem = GTFW.PostWepFix(baseitem,this)
	return baseitem
}

::CTFPlayer.GetWeaponTableByID <- function(idx=null)
{
	if ( type ( idx ) != "integer" ) {
		GTFW.DevPrint(1,"GetWeaponTableByID", "Function failed. Parameter is not an integer. Returning null.")
		return null
	}
	
	// this one catches item IDs if we are using a handle
	local baseitem = GetWeaponTableNoPlayer(idx)
	
	if ( !baseitem ) {
		GTFW.DevPrint(1,"GetWeaponTableByID", "Function failed. Could not find weapon by Item Definition Index. Returning null.")
		return null
	}
	
	baseitem = GTFW.PostWepFix(baseitem,this)
	return baseitem
}








//-----------------------------------------------------------------------------
// Purpose: Filter for all the above functions.
//			Finds a weapon based on handle, slot, ID, classnames, or string.
// 			Requires player handle to start the function.
//-----------------------------------------------------------------------------
::CTFPlayer.GetWeaponTable <- function(weapon=null)
{
	if ( weapon == null ) {
		GTFW.DevPrint(1,"GetWeaponTable", "Function failed. Parameter is Null. Returning null.");
		return null;
	}
	else 
		GTFW.DevPrint(0,"GetWeaponTable", "Getting table for \x22"+weapon+"\x22.");

	
	local ply = this;
	local getWeapon;
	local type = type(weapon);

	switch (type) {
		case "integer" : { getWeapon = ply.GetWeaponTableBySlot(weapon); if (!getWeapon) getWeapon = ply.GetWeaponTableByID(weapon); break; }
		case "string" : { getWeapon = ply.GetWeaponTableByString(weapon); if (!getWeapon) getWeapon = ply.GetWeaponTableByClassname(weapon); break; }
		case "instance" : { getWeapon = ply.GetWeaponTableByHandle(weapon); break; }
		default : { break; }
	}
	
//Error if nothing found
	if ( !getWeapon ) {
		GTFW.DevPrint(1,"GetWeaponTable", "Function failed. Could not find \x22"+weapon+"\x22. Returning null.");
		return null;
	}
	
//
	GTFW.DevPrint(0,"GetWeaponTable", "Found table for \x22"+weapon+"\x22!");
	return getWeapon;
}




	
//-----------------------------------------------------------------------------
// Purpose: Searches all items under tables.nut, then returns a table
// 			First looks through all custom weapons that were registered
// 			Same as handle.GetWeaponTable() but without the handle and post fixes to classnames/ammo.
// 			Used by handle.GetWeaponTable().
//-----------------------------------------------------------------------------
::GetWeaponTableNoPlayer <- function(baseitem=null)
{
	if ( baseitem == null ) {
		GTFW.DevPrint(1,"GetWeaponTableNoPlayer", "Function failed.");
		return;
	}
	
//Define variables
	local typeItem = type ( baseitem );
	local baseitem_toupper;
	if ( typeItem == "string" )
		baseitem_toupper = baseitem.toupper();
	
	local Continue = true
	local ID;
	local wepTable;
	
	local classname;
	local itemid;
	local itemstring;
	local itemstring2;
	
	foreach (table in TF_CUSTOM_WEAPONS_REGISTRY)
	{
		itemstring	= table.name;
		itemid		= table.itemID;
	
	//Searches for CW name first
		if ( typeItem == "string"
		 && itemstring.toupper() == baseitem_toupper )
		{
			ID = itemid;
			wepTable = GTFW.PreWepFix(table, ID);
			
			Continue = false;
			return wepTable;
		}
	//Next searches for item item definition index
		else if ( itemid == baseitem )
		{
			ID = itemid;
			wepTable = GTFW.PreWepFix(table, ID);
				
			Continue = false;
			return wepTable;
		}
	}
	if ( Continue )
	{
		foreach (table in TF_WEAPONS_ALL)
		{
			classname	= table.classname;
			itemid		= table.itemID;
			itemstring = "";
			if ( table.name ) {
				itemstring	= table.name.toupper();
			}
			itemstring2 = "";
			if ( table.name2 ) {
				itemstring2	= table.name2.toupper();
			}
			
	//Searches for classname first, due to things like "revolver" and "REVOLVER" potentially processing as the same thing
			if ( typeItem == "string"
			 && table.classname == baseitem )
			{
				ID = itemid;
				
				wepTable = GTFW.PreWepFix(table, ID);
				
				Continue = false;
				return wepTable;
			}
	// Searches for item string next
			if ( typeItem == "string"
			 && itemstring == baseitem_toupper || itemstring2 == baseitem_toupper )
			{
				ID = itemid;
				
				wepTable = GTFW.PreWepFix(table, ID);
				
				Continue = false;
				return wepTable;
			}
	// then tries item definition index
			if ( itemid == baseitem )
			{
				ID = itemid;
				
				wepTable = GTFW.PreWepFix(table, ID);
				
				Continue = false;
				return wepTable;
			}
		}
	//Searches for warpaints n bot killers last
		if ( Continue )
		{
			foreach (table in TF_WEAPONS_ALL_WARPAINTSnBOTKILLERS)
			{
				if ( table.itemID == baseitem || table.itemID2 == baseitem || table.itemID3 == baseitem
				|| table.itemID4 == baseitem || table.itemID5 == baseitem || table.itemID6 == baseitem
				|| table.itemID7 == baseitem || table.itemID8 == baseitem || table.itemID9 == baseitem
				|| table.itemID10 == baseitem || table.itemID11 == baseitem || table.itemID12 == baseitem
				|| table.itemID13 == baseitem || table.itemID14 == baseitem || table.itemID15 == baseitem)
				{
					if ( table.itemID == baseitem )
						ID = table.itemID;
					else if ( table.itemID2 == baseitem )
						ID = table.itemID2;
					else if ( table.itemID3 == baseitem )
						ID = table.itemID3;
					else if ( table.itemID4 == baseitem )
						ID = table.itemID4;
					else if ( table.itemID5 == baseitem )
						ID = table.itemID5;
					else if ( table.itemID6 == baseitem )
						ID = table.itemID6;
					else if ( table.itemID7 == baseitem )
						ID = table.itemID7;
					else if ( table.itemID8 == baseitem )
						ID = table.itemID8;
					else if ( table.itemID9 == baseitem )
						ID = table.itemID9;
					else if ( table.itemID10 == baseitem )
						ID = table.itemID10;
					else if ( table.itemID11 == baseitem )
						ID = table.itemID11;
					else if ( table.itemID12 == baseitem )
						ID = table.itemID12;
					else if ( table.itemID13 == baseitem )
						ID = table.itemID13;
					else if ( table.itemID14 == baseitem )
						ID = table.itemID14;
					else if ( table.itemID15 == baseitem )
						ID = table.itemID15;
					
					wepTable = GTFW.PreWepFix(table, ID);
					
					return wepTable;
				}
			}
		}
	}
	
	GTFW.DevPrint(1,"GetWeaponTableNoPlayer", "Function failed.");
	return null
}
