//-----------------------------------------------------------------------------
// Purpose: Updates the player's firstperson class arms to use something different.
//			When `saved` set to `true`:
//				- Stores model path+tfclass you set it to
//				- allows loading via GTFW_RestoreCustomClassArms(),
//				- Can only be removed via DeleteCustomClassArms().
// Warning: Will make ent count high if used for a lot of players!
//			This updates entire player loadout with 2 ents per non-custom weapon (one for worldmodel and one for viewmodel).

// Accepts only strings for model path.
// (try using MvM robot world models as class arms, it's fun)
//-----------------------------------------------------------------------------

::CTFPlayer.SetCustomClassArms <- function(arms_model=null, saved=false)
{
	if ( arms_model == null || type( arms_model ) != "string" ) {
		return GTFW.DevPrint(1,"SetCustomClassArms", "Function failed. Parameter is needs string.");
	}
	
	local player = this;
	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	main_viewmodel.ValidateScriptScope();
	local VM_Scope = main_viewmodel.GetScriptScope();
	
// Classarms model is within main_viewmodel's scope
// If there is already a model, delete it, then remake the new one.
	if ( "classarms" in VM_Scope ) {
		local arms = VM_Scope.classarms
		if ( arms && arms.IsValid() ) {
			GTFW.DevPrint(0,"SetCustomClassArms", "DELETED arms " + arms);
			arms.Kill()
		}
		delete VM_Scope["classarms"]
	}
	if ( "GTFW_cleanup" in VM_Scope ) {
		local items = VM_Scope.GTFW_cleanup
		foreach(k, item in items)
			if (item && item.IsValid())
				item.Kill();
		
		delete VM_Scope.GTFW_cleanup
	}
	if ( !( "classarms" in VM_Scope ) ) {
		local classarms_model = player.CreateCustomWearableViewModel(arms_model, main_viewmodel);
		if ( !("GTFW_cleanup" in VM_Scope) ) VM_Scope.GTFW_cleanup <- [];
		VM_Scope.GTFW_cleanup.append(classarms_model);
		
		GTFW.DevPrint(0,"SetCustomClassArms", "Created arms " + arms_model);

		VM_Scope.classarms <- classarms_model;
		
	//if `saved` true, add it to scope
	//if anything but true, remove it from scope (if it exists)
		local playerTFclass = player.GetPlayerClass();
		if ( !("model_path" in VM_Scope) ) VM_Scope["model_path"] <- {};
		
		if ( saved ) {
			VM_Scope["model_path"][playerTFclass] <- arms_model;
		}
		else {
			VM_Scope["model_path"][playerTFclass] <- null;
		}
	}
	
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = GetPropEntityArray(player, "m_hMyWeapons", i)
		
		if ( wep == null ) continue
		wep.ValidateScriptScope();
		local wep_scope = wep.GetScriptScope();
	
	//resets custom status only if SetCustomClassArms() was run once before.
	// had to do it this way for GTFW_Cleanup() to work; weapons that were set custom by this would never pass here.
		if ( "using_custom_arms" in wep_scope ) {
			IS_CUSTOM(wep,false);
		}
		if ( !(GetItemID(wep) & ~((1<<16)-1) ) ) IS_CUSTOM(wep,false);
		if ( IS_CUSTOM(wep) ) continue
		IS_CUSTOM(wep,true);
		wep_scope["using_custom_arms"] <- true;
		local modelindex = GetPropInt(wep, "m_iWorldModelIndex");
		GTFW.UpdateCustomWeapon(player, wep, modelindex,modelindex)
		GTFW.DevPrint(0,"SetCustomClassArms", "Updated" + wep + " with new classarms!");
	}
	
	GTFW.AddThinkToViewModel(player, player.GetActiveWeapon());
}


//-----------------------------------------------------------------------------
// Purpose: Restores classarms for player class if used SetCustomClassArms() prior.
// 			Meant for OnGameEvent_post_inventory_application()
//-----------------------------------------------------------------------------

::CTFPlayer.GTFW_RestoreCustomClassArms <- function()
{
	local player = this;
	
	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	if ( !main_viewmodel || !main_viewmodel.IsValid() ) return;
	
	main_viewmodel.ValidateScriptScope();
	local VM_Scope = main_viewmodel.GetScriptScope();
	if ( "model_path" in VM_Scope ) {
		local tfclass = player.GetPlayerClass();
		local classarms_modelpath;
		
		local vmPaths = VM_Scope["model_path"];
		foreach ( playerTFclass, arms_model in vmPaths ) {
			if ( tfclass == playerTFclass ) {
				classarms_modelpath = arms_model;
				break;
			}
		}
		
		if ( classarms_modelpath ) {
			if ( classarms_modelpath != GTFW_MODEL_ARMS[player.GetPlayerClass()] ) {
				player.SetCustomClassArms(classarms_modelpath);
			}
		}
		else if ( tfclass in vmPaths ) {
			player.DeleteCustomClassArms(tfclass);
			player.SwitchToBest(player.GetActiveWeapon());
			delete VM_Scope["model_path"][tfclass]
		}
	}
}

//-----------------------------------------------------------------------------
// Purpose: Deletes custom classarms if for player if used SetCustomClassArms() prior.
//			has option to remove specific tfclass' custom arms, instead of all of them
// Warning: What this does not do is clear the custom flags off weapons that weren't custom to begin with. Use Regenerate() to fix that.
//-----------------------------------------------------------------------------

::CTFPlayer.DeleteCustomClassArms <- function(specific_playerclass=0)
{
	local player = this;
	
	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	
	if ( !main_viewmodel || !main_viewmodel.IsValid() ) return;
	
	main_viewmodel.ValidateScriptScope();
	local VM_Scope = main_viewmodel.GetScriptScope();
	if ( "model_path" in VM_Scope ) {
		local vmPaths = VM_Scope.model_path;
		if ( specific_playerclass > 0 && specific_playerclass < 11) {
			foreach ( playerTFclass, arms_model in vmPaths ) {
				if ( specific_playerclass == playerTFclass ) {
					VM_Scope["model_path"][playerTFclass] <- null;
					break;
				}
			}
		}
		else 
			delete VM_Scope.model_path;
	}
	
	local using_a_custom_wep = [];
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = GetPropEntityArray(player, "m_hMyWeapons", i)
		
		if ( wep == null ) continue
		wep.ValidateScriptScope();
		local wep_scope = wep.GetScriptScope();
		if ( "using_custom_arms" in wep_scope ) delete wep_scope["using_custom_arms"];
		if ( "from_GiveWeapon" in wep_scope && wep_scope.from_GiveWeapon == false ) IS_CUSTOM(wep,false);
		if ( IS_CUSTOM(wep) ) using_a_custom_wep.append(wep);
		
		GTFW.DevPrint(0,"SetCustomClassArms", "Removed classarms data from" + wep);
	}
	
	if ( "classarms" in VM_Scope ) {
		GTFW.DevPrint(0,"RemoveCustomClassArms", "validating");
		local classarms = VM_Scope.classarms;
	  
		if ( classarms && classarms.IsValid() ) {
			GTFW.DevPrint(0,"RemoveCustomClassArms", "restoring class arms");
			main_viewmodel.EnableDraw();
			classarms.Kill();
		}
		
		if ( using_a_custom_wep.len() == 0 ) {
			delete VM_Scope["classarms"];
			return;
		}
		local tfclass = player.GetPlayerClass();
		local arms;
		if ( player.HasGunslinger() && tfclass == TF_CLASS_ENGINEER )
			arms = GTFW_MODEL_ARMS[10];
		else 
			arms = GTFW_MODEL_ARMS[tfclass];
		local classarms_model = player.CreateCustomWearableViewModel(arms, main_viewmodel);

		GTFW.DevPrint(0,"RemoveCustomClassArms", "Restored arms " + classarms_model);
		
		main_viewmodel.DisableDraw();
		
	// Purpose: Puts bonemerged class arms model in main_viewmodel scope for easy tracking
		VM_Scope["classarms"] <- classarms_model;
	}
	
}