//-----------------------------------------------------------------------------
//	Purpose: Spawns in a weapon pickup that any class can pickup
//-----------------------------------------------------------------------------

function SpawnWeaponProp(player, hWeapon, origin, anglez=QAngle(0,0,0), velocity = Vector(0, 0, 350), filermode = false, despawnTimer=30) {
	
	local hWeaponScope, modelPath, table;
	if ( type( hWeapon ) == "instance" && hWeapon.IsValid() )
	{
		hWeapon.ValidateScriptScope();
		hWeaponScope = hWeapon.GetScriptScope();
		modelPath = "worldModelProp" in hWeaponScope 
		if ( modelPath ) modelPath = hWeaponScope.worldModelProp;
		if ( !modelPath || !modelPath.IsValid() ) modelPath = GetItemModelName(GetItemID(hWeapon));
		else modelPath = modelPath.GetModelName();
	}
	else {
		hWeaponScope = player.GetWeaponTable(hWeapon);
		if ( hWeaponScope == null ) return;

		if ( type(hWeaponScope.worldModel) == "string" && hWeaponScope.worldModel.len() ) modelPath = hWeaponScope.worldModel;
		else modelPath = GetItemModelName(hWeaponScope.itemID);
	}
	
	if ( type(modelPath) != "string" || !modelPath.len() ) return;
	
	PrecacheModel(modelPath);
	local physicsWeapon = SpawnEntityFromTable("prop_physics_override", {
		angles = anglez,
		spawnflags = 8192,
		effects = 4,
		skin = player.GetTeam() == 3 ? 1 : 0,
		model = modelPath,
		origin = origin,
		"OnUser1#1" : format("%s%i%s","!self,Kill,,",despawnTimer,",-1")
	});
	physicsWeapon.SetModelSimple(modelPath)
	physicsWeapon.ApplyAbsVelocityImpulse(velocity);
	physicsWeapon.AcceptInput("FireUser1", "", null, null);	//fires I/O OnUser1 to enable despawn timer
	
	physicsWeapon.SetCollisionGroup(COLLISION_GROUP_PUSHAWAY)
	
	local players_last = {};
	
	physicsWeapon.ValidateScriptScope();
	local physicsWeaponScope = physicsWeapon.GetScriptScope();
	physicsWeaponScope.pickupThink <- function()
	{
		local nearby_players = {};
		for(local ply = null; ply = Entities.FindByClassnameWithin(ply,"player",physicsWeapon.GetCenter(), 128.0); ) {
			nearby_players[ply] <- true;
			if ( !( ply in players_last ) ) {
				local glow = SpawnPlayerOnlyGlow(ply,physicsWeapon)
				players_last[ply] <- glow;
			//	printl("added " + ply);
			}
		}

		foreach (k,v in players_last) {
			if ( !( k in nearby_players ) ) {
				local glow = v;
				if ( glow && glow.IsValid() ) glow.Kill();
				
				delete players_last[k]
			//	printl("removed " + k)
			}
			else {
				Looking(k, physicsWeapon, hWeaponScope)
			}
		}
		
		return 0.0
	}

	EntFireByHandle(physicsWeapon,"RunScriptCode", "AddThinkToEnt(self, \x22pickupThink\x22)",0.33,null,null)
	
	return;
}
/*
	local min = physicsWeapon.GetBoundingMins()*5;
	local max = physicsWeapon.GetBoundingMaxs()*5;
	local trigger = SpawnEntityFromTable("trigger_multiple", {
		mins = min,
		maxs = max,
		origin = physicsWeapon.GetOrigin(),
		spawnflags = 1,
		StartDisabled = 0,
		wait	= 0.03,
	})

	SetPropVector(trigger, "m_Collision.m_vecMinsPreScaled", min)
	SetPropVector(trigger, "m_Collision.m_vecMins", min)

	SetPropVector(trigger, "m_Collision.m_vecMaxsPreScaled", max)
	SetPropVector(trigger, "m_Collision.m_vecMaxs", max)

	trigger.ValidateScriptScope()
	trigger.GetScriptScope().glow <- []
	
	EntityOutputs.AddOutput(trigger, "OnStartTouch", "!self", "RunScriptCode", "self.GetScriptScope().glow.append(SpawnPlayerOnlyGlow(activator,self.GetRootMoveParent()))", 0.0, -1);
	EntityOutputs.AddOutput(trigger, "OnEndTouch", "!self", "RunScriptCode", "foreach( k in self.GetScriptScope().glow) if (k.IsValid()) k.Kill()", 0.0, -1);
	trigger.AcceptInput("SetParent", "!activator", physicsWeapon, physicsWeapon);
	
	EntityOutputs.AddOutput(trigger, "OnTrigger", "!self", "RunScriptCode", format("Looking(activator,self.GetRootMoveParent(),%i)",hWeaponScope.itemID), 0.0, -1);
	EntityOutputs.AddOutput(trigger, "OnTrigger", "!self", "RunScriptCode", "ApplyAbsVelocityImpulse(Vector(0,0,0))", 0.0, -1);
	//EntityOutputs.AddOutput(trigger, "OnStartTouch", "!activator", "RunScriptCode", "self.TestPlayerGiveWeaponAndSave("+hWeaponScope.itemID+", caller)", 0.0, -1);
	trigger.SetSolid(SOLID_BBOX);
*/


function Looking(player, item, weaponParams) {

	if ( !item || !item.IsValid() ) return;
	
	local table = {
		start = player.EyePosition() + player.GetForwardVector(),	//start position is player's eye position
		end = player.EyePosition()+(player.EyeAngles().Forward()*9999),	//end position is coordinate player eye position + (where they're facing*9999 distance)
		ignore = player,
	}
	
	TraceLineEx(table)
	
	if ( "enthit" in table ) {
		if ( table.enthit == item ) {
		//	DebugDrawLine(player.EyePosition(),item.GetCenter(),0,255,0,false,3)
			if ( player.IsUsingActionSlot() ) {
				item.Kill()
				SpawnWeaponProp(player, player.GetWeaponBySlot(weaponParams.slot), player.GetCenter(), player.GetAbsAngles(), player.GetAbsVelocity());
				player.GiveWeapon(weaponParams.itemID);
			}
		}
	}
//	else DebugDrawLine(player.EyePosition(),item.GetCenter(),255,0,0,false,3)
	return null;
}

function SpawnPlayerOnlyGlow(player, weapon)
{
	local proxy_entity = Entities.CreateByClassname("obj_teleporter") // not using SpawnEntityFromTable as that creates spawning noises
	proxy_entity.SetAbsOrigin(weapon.GetOrigin())
	proxy_entity.DispatchSpawn()
	
	PrecacheModel("models/empty.mdl")
	proxy_entity.SetModel("models/empty.mdl")
	
	local modelpath = weapon.GetModelName(); //"models/empty.mdl"
	PrecacheModel(modelpath)
	proxy_entity.SetModel(modelpath)
	proxy_entity.SetAbsOrigin(weapon.GetOrigin())
	proxy_entity.SetAbsAngles(weapon.GetAbsAngles())
	proxy_entity.SetSkin(weapon.GetSkin())
	
//	proxy_entity.KeyValueFromInt("effects",129);
	proxy_entity.AddEFlags(Constants.FEntityEFlags.EFL_NO_THINK_FUNCTION) // prevents the entity from disappearing
	proxy_entity.SetSolid(Constants.ESolidType.SOLID_NONE)
	SetPropBool(proxy_entity, "m_bPlacing", true)
	SetPropInt(proxy_entity, "m_fObjectFlags", 2) // sets "attachment" flag, prevents entity being snapped to player feet

	// m_hBuilder is the player who the entity will be networked to only
	SetPropEntity(proxy_entity, "m_hBuilder", player)
	// set glow enabled for teleporter
	SetPropInt(proxy_entity,"m_bGlowEnabled",1);
	proxy_entity.AcceptInput("SetParent", "!activator", weapon, weapon);
	return proxy_entity;
/*	
//	Testing tf_glow, didn't work for me.
	proxy_entity.KeyValueFromString("targetname", UniqueString());
	local glow = SpawnEntityFromTable("tf_glow", {
		GlowColor = "190 197 40 255",
		target = proxy_entity.GetName(),
	});
//	SetPropEntity(glow, "m_hTarget", proxy_entity)
	glow.DispatchSpawn()
	
	glow.AcceptInput("SetParent", "!activator", proxy_entity, proxy_entity);
	proxy_entity.AcceptInput("SetParent", "!activator", weapon, weapon);
	
	return proxy_entity;
*/
}

//-----------------------------------------------------------------------------
//	Purpose: Spawns in a tf_dropped_weapon you can pickup.
//-----------------------------------------------------------------------------
/*
function SpawnWeaponPro2p(hWeapon, origin, anglez=QAngle(0,0,0), despawnTimer=30) {
//	local hWeapon = hPlayer.GetActiveWeapon();
	if ( !hWeapon || !hWeapon.IsValid() ) return
	
	hWeapon.ValidateScriptScope();
	local hWeaponScope = hWeapon.GetScriptScope();
	local modelPath = "worldModelProp" in hWeaponScope 
	if ( modelPath ) modelPath = hWeaponScope.worldModelProp;
	if ( !modelPath || !modelPath.IsValid() ) modelPath = GetItemModelName(GetItemID(hWeapon));
	else modelPath = modelPath.GetModelName();
	
	if ( type(modelPath) == "string" && modelPath.len() ) {
		PrecacheModel(modelPath);
		local physicsWeapon = SpawnEntityFromTable("tf_dropped_weapon", {
			angles = anglez,
			spawnflags = 4,
			effects = 4,
			skin = hWeapon.GetTeam() == 3 ? 1 : 0,
			model = modelPath,
			origin = origin,
			"OnUser1#1" : format("%s%i%s","!self,Kill,,",despawnTimer,",-1")
		});
		physicsWeapon.SetTeam(hWeapon.GetOwner().GetTeam());
		SetPropInt(physicsWeapon, "m_Item.m_iItemDefinitionIndex", GetItemID(hWeapon));
		SetPropBool(physicsWeapon, "m_Item.m_bInitialized", true);
		physicsWeapon.SetModelSimple(modelPath)
	//	physicsWeapon.DispatchSpawn();
		printl(modelPath);
		physicsWeapon.ApplyAbsVelocityImpulse(Vector(0, 0, 350));
		physicsWeapon.AcceptInput("FireUser1", "", null, null);	//fires I/O OnUser1 to enable despawn timer
	}
}


function SpawnWeaponPro3p(player,hWeapon)
{
	if ( !hWeapon || !hWeapon.IsValid() ) return
	hWeapon.ValidateScriptScope();
	local hWeaponScope = hWeapon.GetScriptScope();
	local modelPath = "worldModelProp" in hWeaponScope 
	if ( modelPath ) modelPath = hWeaponScope.worldModelProp;
	if ( !modelPath || !modelPath.IsValid() ) modelPath = GetItemModelName(GetItemID(hWeapon));
	else modelPath = modelPath.GetModelName();
	
	//for ( local weapon; weapon = Entities.FindByClassname(weapon, "tf_dropped_weapon"); ) {
	//if ( weapon == null ) continue
	local weapon = Entities.FindByClassnameNearest("tf_dropped_weapon", player.GetOrigin(), 128.0)
	if ( weapon && weapon.IsValid() ) {
		printl(weapon + " " + GetPropInt(weapon,"m_Item.m_iItemDefinitionIndex"))
		weapon.KeyValueFromString("classname", "dummy")
		SetPropInt(weapon, "m_Item.m_iItemDefinitionIndex", GetItemID(hWeapon));
		weapon.SetModelSimple(modelPath)
		local proxy_entity = SpawnPlayerOnlyGlow(player, weapon)
	}
}
function dropped_weapon_think() {}
*/