//-----------------------------------------------------------------------------
// Purpose: Sets bodygroup of bonemerged class model
//		 ** MAKE SURE THIS IS CALLED BEFORE SetCustomModelWithClassAnimations()!! 
//		 ** Otherwise player might be a different class model from bonemerged (thus breaking this function)
//-----------------------------------------------------------------------------

::GTFW.SetBodygroupsForTFClassModel <- function(player, playermodel)
{
	local playerscope = player.GetScriptScope();
	local bodygroups = playerscope.GTFW_playerModel_Bodygroups
	for (local i = 0; i < 10; i++) {
		if ( bodygroups.len() == i ) break;
		//if ( playermodel.GetBodygroupName(i) == "" ) break;
	//	printl(bodygroups[i]);
		playermodel.AcceptInput("RunScriptCode",format("self.SetBodygroup(%i,%i)",i,bodygroups[i]), playermodel, playermodel)
	}
}

//-----------------------------------------------------------------------------
// Purpose: Sets custom class animations without it looking glitchy
//-----------------------------------------------------------------------------
::GTFW.SetCustomModelWithClassAnimations <- function(player, modelpath) {
	player.ValidateScriptScope()
	local playerscope = player.GetScriptScope();
	
	local classAnimsModel;
	if ( "GTFW_classAnimsModel" in playerscope ) {
		classAnimsModel = playerscope.GTFW_classAnimsModel;

		if ( !classAnimsModel || !classAnimsModel.IsValid() ) player.SetCustomModelWithClassAnimations(modelpath)
		else if ( "GTFW_rendermode" in playerscope && playerscope.GTFW_rendermode == false ) {
			player.SetCustomModelWithClassAnimations(modelpath);
			SetPropInt(classAnimsModel, "m_nModelIndexOverrides", GetModelIndex(modelpath));
			classAnimsModel.SetModelSimple(modelpath)
		}
		else { 
			SetPropInt(classAnimsModel, "m_nModelIndexOverrides", GetModelIndex(modelpath));
			classAnimsModel.SetModelSimple(modelpath)
		}
	}
	else player.SetCustomModelWithClassAnimations(modelpath)
	
	playerscope.GTFW_playerOriginalClassModel <- modelpath
	
}

//-----------------------------------------------------------------------------
// Purpose: Creates and bonemerges a player class model. Meant for players changing animations while using weapons unintended for the class.
//			Accepts only player's handle.
//			(as far as 5.4.0 this is only used in AddThinkToViewModel() under "GiveWeapon().nut" )
//-----------------------------------------------------------------------------

::GTFW.CreateTFClassModel <- function(player, classAnimsModel)
{
	if ( !player.IsPlayer() ) {
		return GTFW.DevPrint(1,"CreateTFClassModel", "Function failed. Parameter is needs player handle.");
	}
	
	player.ValidateScriptScope()
	local playerscope = player.GetScriptScope();
	playerscope.classAnimsModel <- player.GetModelName();
	
	if ( "GTFW_classAnimsModel" in playerscope ) {
		local playermodel = playerscope["GTFW_classAnimsModel"];
		if ( playermodel && playermodel.IsValid() ) {
			playermodel.Kill()
		}
		delete playerscope["GTFW_classAnimsModel"];
	}
	
	if ( classAnimsModel = null ) classAnimsModel = GTFW_MODEL_TFCLASSES[player.GetPlayerClass()];
	local playermodel = player.CreateCustomWearable(classAnimsModel);
	playermodel.DisableDraw();
	
	
	playerscope.GTFW_playerModel_Bodygroups <- [];
	for (local i = 0; i < 10; i++) {
		if ( player.GetBodygroupName(i) == "" ) break;
		playerscope.GTFW_playerModel_Bodygroups.append(player.GetBodygroup(i));
	}
	GTFW.SetBodygroupsForTFClassModel(player, playermodel);
	
	playerscope.GTFW_classAnimsModel <- playermodel;
	playerscope.GTFW_playerOriginalClassModel <- classAnimsModel;
	
	playermodel.ValidateScriptScope()
	playermodel.GetScriptScope().classAnimsModel <- classAnimsModel
	
	GTFW.DevPrint(0,"CreateTFClassModel", "Created player model " + playermodel);
	
	playerscope.GTFW_rendermode <- false;
	
	
	return playermodel;
}


//-----------------------------------------------------------------------------
// Enables / Disables class model in AddThinkToView()
//-----------------------------------------------------------------------------

::GTFW.SetUseClassAnimations_ON <- function(player, playerscope, playermodel=null, wepScope={ tf_class = 0 }, classAnimsModel="models/bots/engineer/bot_engineer.mdl")
{

	playerscope.GTFW_rendermode <- true;
	
	local color = GetPropInt(player, "m_clrRender");
	SetPropInt(player, "m_clrRender", color & ~(0xFF << 24) );
	if (playermodel) {
		GTFW.SetBodygroupsForTFClassModel(player, playermodel);
		
		if ( classAnimsModel == "" )
			classAnimsModel = GTFW_MODEL_TFCLASSES[player.GetPlayerClass()];
		else if ( wepScope.tf_class == 0 )
			GTFW_MODEL_TFCLASSES[wepScope.tf_class];
		
		player.SetCustomModelWithClassAnimations(classAnimsModel);
		SetPropInt(playermodel, "m_nRenderMode", 0);
		SetPropInt(playermodel, "m_fEffects", 129);
		SetPropInt(playermodel, "m_clrRender", color | (0xFF << 24) );
		playermodel.AcceptInput("EnableShadow", "", null, playermodel);
	}
}

::GTFW.SetUseClassAnimations_OFF <- function(player, playerscope, playermodel=null, wepScope={ tf_class = 0 }, classAnimsModel="models/bots/engineer/bot_engineer.mdl")
{
//printl("OFF");
//printl(classAnimsModel)
	playerscope.GTFW_rendermode <- false;
	
	local color = GetPropInt(player, "m_clrRender");
	SetPropInt(player, "m_clrRender", color | (0xFF << 24) );
	if (playermodel) {
		GTFW.SetBodygroupsForTFClassModel(player, playermodel);
	
		if ( classAnimsModel == "" )
			classAnimsModel = GTFW_MODEL_TFCLASSES[player.GetPlayerClass()];
		else classAnimsModel = playermodel.GetModelName();
	
		player.SetCustomModelWithClassAnimations(classAnimsModel);
		SetPropInt(playermodel, "m_nRenderMode", 10);
		SetPropInt(playermodel, "m_fEffects", 161);
		SetPropInt(playermodel, "m_clrRender", color & ~(0xFF << 24) );
		playermodel.AcceptInput("DisableShadow", "", null, playermodel);
	}
}