/*-----------------------------------------------------------------------------
	give_tf_weapon, or "GTFW" for short,
	Version 5.4.3
	By Yakibomb
*///---------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	Weapon Config for GiveWeapon() and RegisterCustomWeapon()
//-----------------------------------------------------------------------------
// These are settings that are used by GTFW.
// You can preset the defaults for every weapon spawned by GiveWeapon(), or on a per-weapon basis as needed.
// For per-weapon, use parameter 3 in GiveWeapon() and add the bits you want, or simply make a custom weapon with a different preset.

// `Defaults` preset for all weapons is below this list.

// Here are all the properties you can use:

::DeleteAndReplace <-	(1 << 0)	// Deletes the weapon that matches the slot of the new weapon, then adds the new weapon. Not compatible with KeepIDX bit. NOTE: Cannot switch to another weapon in the same slot, unless using "hud_fastswitch 0".
::KeepIDX <-			(1 << 1)	// Only updates the Item Definition Index of the given weapon. Not compatible with DeleteAndReplace bit. Added for MvM to allow for custom weapons to be upgradeable.
::AutoSwitch <-			(1 << 2)	// Forcefully switches to the weapon if obtained.
::WipeAttributes <-		(1 << 3)	// Clears original attributes present on the weapon.
::ForceCustom <-		(1 << 4)	// Forces the weapon to be "custom". "custom" in GTFW terms means different things, but here it means it sets netprop "ItemIDHigh" bit 6, for coding convenience. (The other "custom" means unique ItemDefinitionIndex. Don't worry about it right now)
::ForceNotCustom <-		(1 << 5)	//  Makes sure to unset "ItemIDHigh" bit 6.
::AnnounceToChat <- 	(1 << 6)	// Announces the weapon in chat for all to see what you got!
::Save <- 				(1 << 7)	// Saves the weapon, allowing players to retrieve it as long as function "handle.LoadLoadout()" is used.
::AutoRegister <- 		(1 << 8)	// This automatically registers this as a custom weapon, if it wasn't already registered before. It might suck up all the unique ItemDefinitionIndexes, so use with caution, or don't use this setting at all.
::FixClassAnims <- 		(1 << 9)	// Player model switches to class animations of weapon's intended player class. No disgression--If it the weapon is unintended for that class, it will switch.
::FixClassAnimsRef <-	(1 << 10)	//  Same as FixClassAnims, but only for when player goes into reference pose.
::FixClassAnimsITEM <-	(1 << 11)	//  Same as FixClassAnims, but only for unique ITEM animations (ITEM1, ITEM2, ITEM3, ITEM4).

//-----------------------------------------------------------------------------
// This is the Defaults preset for all weapons. Adjust it how you like.
//-----------------------------------------------------------------------------
//Defaults: DeleteAndReplace|AutoSwitch|Save|FixClassAnimsRef|FixClassAnimsITEM
//To remove all, write:
//::Defaults <- 0

::Defaults <- DeleteAndReplace|AutoSwitch|Save


//-----------------------------------------------------------------------------
//	VScript CVARs
//-----------------------------------------------------------------------------
Convars.SetValue("tf_dropped_weapon_lifetime", 0)	//disables dropped weapons because they're buggy with this script
::CVAR_GTFW_DEBUG_MODE <- false		// Sends error messages to everyone. False by default.

::CVAR_USE_VIEWMODEL_FIX <- true				// Automatically fixes any and all viewmodel arms to match the class you're playing as. True by default.
::CVAR_DELETEWEAPON_AUTO_SWITCH <- true			// Automatically switches weapon to another if deleting a weapon. True by default.
::CVAR_DISABLEWEAPON_AUTO_SWITCH <- true		// Automatically switches weapon to another if disabling a weapon. True by default.
::CVAR_ENABLEWEAPON_AUTO_SWITCH <- false		// Automatically switches weapon to another if re-enabling a weapon. False by default.

const GLOBAL_WEAPON_COUNT = 8	//How many loops the script makes to check for unused slots

//-----------------------------------------------------------------------------
//	Included Scripts
//-----------------------------------------------------------------------------
// Only runs the main code files once to improve server performance.

//If TF_CUSTOM_WEAPONS_REGISTRY is already defined, stop running the script. Otherwise, continue to execute the main code files.
::ROOT <- getroottable();
//if ( "TF_CUSTOM_WEAPONS_REGISTRY" in ROOT ) return;
if ( !( "GTFW" in ROOT ) ) ::GTFW <- {};

::TF_CUSTOM_WEAPONS_REGISTRY <- {}; // Weapon's registry only allows for registering same weapon type 63 times.

IncludeScript("stopthattank/give_tf_weapon/__exec.nut")			// Executes all functions that this script uses!